@@ -4,14 +4,38 @@ use warnings;
 use strict;
 use Carp();
 
+use List::Util();
 use CGI::FormBuilder 3;
 
 # C::FB sometimes gets confused when passed CDBI::Column objects as field names, 
 # hence all the map {''.$_} column filters. Some of them are probably unnecessary, 
 # but I need to track down which.
 
-our $VERSION = 0.3;
+our $VERSION = '0.31';
 
+our @BASIC_FORM_MODIFIERS = qw( hidden options file );
+
+our %ValidMap = ( varchar   => 'VALUE',
+                  char      => 'VALUE', # includes MySQL enum and set
+                  blob      => 'VALUE', # includes MySQL text
+                  text      => 'VALUE',
+                  
+                  integer   => 'INT',
+                  bigint    => 'INT',
+                  smallint  => 'INT',
+                  tinyint   => 'INT',
+                  
+                  date      => 'VALUE',
+                  
+                  # normally you want to skip validating a timestamp column...
+                  #timestamp => 'VALUE',
+                  
+                  double    => 'NUM',
+                  float     => 'NUM',
+                  decimal   => 'NUM',
+                  numeric   => 'NUM',
+                  );    
+                  
 sub import
 {
     my ( $class, %args ) = @_;
@@ -23,6 +47,8 @@ sub import
     my @export = qw( as_form 
                      search_form
                      
+                     as_form_with_related
+                     
                      update_or_create_from_form
                      
                      retrieve_from_form
@@ -157,9 +183,9 @@ itself).
 
 sub as_form
 {
-    my $proto = shift;
+    my ( $proto, %args_in ) = @_;
     
-    my ( $orig, %args ) = __PACKAGE__->_get_args( $proto, @_ );
+    my ( $orig, %args ) = __PACKAGE__->_get_args( $proto, %args_in );
     
     __PACKAGE__->_setup_auto_validation( $proto, \%args );
     
@@ -178,15 +204,15 @@ while processing a form submission:
     
 pk hidden fields are always present in rendered forms, but may be empty (submits undef). undef does not 
 pass validation tests. The solution is to place pk fields in 'keepextras', not in 'fields'. That means they 
-are not validated at all. The only (I think) place pk data are used is in retrieve_from_form
+are not validated at all. The only (I think) place submitted pk data are used is in retrieve_from_form
 
 =cut
 
 sub _get_args
 {
-    my ( $me, $proto ) = ( shift, shift );
+    my ( $me, $proto, %args_in ) = @_;
     
-    my %args = ( %{ $proto->form_builder_defaults }, @_ );
+    my %args = ( %{ $proto->form_builder_defaults }, %args_in );
     
     # take a copy, and make sure not to transform undef into []
     my $original_fields = $args{fields} ? [ @{ $args{fields} } ] : undef;
@@ -195,7 +221,8 @@ sub _get_args
     
     $args{fields} ||= [ map  {''.$_} 
                         grep { ! $pk{ ''.$_ } }    
-                        $proto->columns( 'All' ) 
+                        #$proto->columns( 'All' ) 
+                        $me->_db_order_columns( $proto, 'All' )
                         ];
                         
     $args{keepextras} = [ keys %pk ];
@@ -218,6 +245,20 @@ sub _get_args
     return $orig, %args;
 }
 
+# Get deep into CDBI to extract the columns in the same order as defined in the database.
+# In fact, this returns the columns in the order they were originally supplied to 
+# $proto->columns( All => [ col list ] ) Defaults 
+# to the order returned from the database query in CDBI::Loader, which for MySQL, 
+# is the same as the order in the database.
+sub _db_order_columns 
+{
+    my ( $me, $them, $group ) = @_;
+    
+    $group ||= 'All';
+    
+    return @{ $them->__grouper->{_groups}->{ $group } };
+} 
+
 sub _make_form
 {
     my ( $me, $them, $orig, %args ) = @_;
@@ -227,7 +268,7 @@ sub _make_form
     $form->{__cdbi_original_args__} = $orig;
     
     # this assumes meta_info only holds data on relationships
-    foreach my $modify ( 'hidden', 'options', 'file', keys %{ $them->meta_info } ) 
+    foreach my $modify ( @BASIC_FORM_MODIFIERS, keys %{ $them->meta_info } ) 
     {
         my $form_modify = "form_$modify";
         
@@ -237,6 +278,352 @@ sub _make_form
     return $form;
 }
 
+=item as_form_with_related
+
+Builds a form with fields from the target CDBI class/object, plus fields from the related objects. 
+
+Accepts the same arguments as C<as_form>, with these additions:
+
+=over 4
+
+=item related
+
+A hashref of C<< $field_name => $as_form_args_hashref >> settings. Each C<$as_form_args_hashref> 
+can take all the same settings as C<as_form>. These are used for generating the fields of the class or 
+object(s) referred to by that field. 
+
+=item show_related
+
+By default, all related fields are shown in the form. To only expand selected related fields, list 
+them in C<show_related>.
+
+=back
+
+=cut
+
+sub as_form_with_related
+{
+    my ( $proto, %args ) = @_;
+    
+    my $related_args = delete( $args{related} );
+    my $show_related = delete( $args{show_related} ) || [];
+    
+    my $parent_form = $proto->as_form( %args );
+    
+    foreach my $field ( __PACKAGE__->_fields_and_has_many_accessors( $proto, $parent_form, $show_related ) )
+    {
+        # object or class
+        my ( $related, $rel_type ) = __PACKAGE__->_related( $proto, $field );
+        
+        next unless $related;
+        
+        my @relateds = ref( $related ) eq 'ARRAY' ? @$related : ( $related );
+        
+        __PACKAGE__->_splice_form( $_, $parent_form, $field, $related_args->{ $field }, $rel_type ) for @relateds;
+    }
+    
+    return $parent_form;
+}
+
+# deliberately ugly name to encourage something more generic in future
+sub _fields_and_has_many_accessors
+{
+    my ( $me, $them, $form, $show_related ) = @_;
+    
+    return @$show_related if @$show_related;
+    
+    my @fields = $form->field;
+    
+    my %seen = map { $_ => 1 } @fields;
+    
+    my @related = keys %{ $them->meta_info( 'has_many' ) || {} };
+    
+    push @fields, grep { ! $seen{ $_ } } @related;
+    
+    return @fields;
+}
+        
+sub _splice_form
+{
+    my ( $me, $them, $parent_form, $field_name, $args, $rel_type ) = @_;
+    
+    my $related_form = $them->as_form( %$args );
+    
+    my $moniker = $them->moniker;
+    
+    my @related_fields;
+    
+    foreach my $related_field_name ( map { ''.$_ } $related_form->field )
+    {
+        my $related_field = $related_form->{fieldrefs}{ $related_field_name };
+        
+        my $fake_name = $me->_false_related_field_name( $them, $related_field_name );
+        
+        delete $related_field->{_form};
+        
+        $related_field->{name} = $fake_name; 
+        
+        $related_field->{_form} = $parent_form;
+        
+        $parent_form->{fieldrefs}{ $fake_name } = $related_field;
+    
+        $parent_form->field( name  => $fake_name,
+                             label => ucfirst( $moniker ) . ': ' . $related_field_name,
+                             ) 
+                                unless $args->{labels}{ $related_field_name };
+                                
+        push @related_fields, $related_field;
+    }
+
+    my $offset = 0;
+    
+    foreach my $parent_field ( @{ $parent_form->{fields} } )
+    {
+        $offset++;
+        last if $parent_field->name eq $field_name;        
+    }
+    
+    #push @{ $parent_form->{fields} }, $related_field;
+    splice @{ $parent_form->{fields} }, $offset, 0, @related_fields;
+
+    # different rel_types get treated differently e.g. is_a should probably not 
+    # allow editing
+    if ( $rel_type eq 'has_a' )
+    {
+        $parent_form->field( name => $field_name,
+                             type => 'hidden',
+                             );
+    }
+    elsif ( $rel_type eq 'is_a' )
+    {
+        $parent_form->field( name     => ''.$_,
+                             readonly => 1,
+                             )
+                                for @related_fields;
+    }
+    
+}
+        
+        
+        
+        
+        
+
+
+
+
+=for retiring
+
+# fields keepextras values required
+sub _get_related_args
+{
+    my ( $me, $them, $main_form_args, $related_args_in ) = @_;
+
+    my @main_form_fields = @{ $main_form_args->{fields} };
+    
+    my $offset = 0;
+    
+    my $related_orig = {};
+    
+    foreach my $field ( @main_form_fields )
+    {
+        # object or class
+        my ( $related, $rel_type ) = __PACKAGE__->_related( $them, $field );
+        
+        next unless $related;
+        
+        my $related_class = ref( $related ) || $related;
+        
+        $main_form_args->{_relatives_}->{ $rel_type }->{ $related_class } = [];
+        
+        push( @{ $main_form_args->{_related_}->{ $rel_type }->{ $related_class } }, $related )
+            if ref( $related );
+        
+        my ( $orig, %related_args ) = __PACKAGE__->_get_args( $related, %$related_args_in ); 
+        
+        $related_orig->{ $related_class } = $orig;
+        
+        my @false_fnames = map { __PACKAGE__->_false_related_field_name( $related, $_ ) } 
+                               @{ $related_args{fields} };
+        
+        splice @{ $main_form_args->{fields} }, ++$offset, 0, @false_fnames;
+    
+        if ( ref $related )
+        {
+            my @values = map { '' . $related->get( $_ ) } @{ $related_args{fields} };
+            
+            splice @{ $main_form_args->{values} }, $offset, 0, @{ $related_args{values} };
+        }
+        
+        $main_form_args->{required} ||= [];
+        $related_args{required}     ||= [];
+    
+        push @{ $main_form_args->{required} }, @{ $related_args{required} };
+        
+        $offset += @{ $related_args{fields} };
+    }
+    
+    return ( $related_orig, %$main_form_args );
+}
+
+=cut
+
+sub _related
+{
+    my ( $me, $them, $field ) = @_;
+    
+    my @rel_types = keys %{ $them->meta_info };
+
+    my $related_meta = List::Util::first { $_ } map { $them->meta_info( $_ => $field ) } @rel_types;
+    
+    return unless $related_meta;
+
+    my $rel_type = $related_meta->name;
+                  
+    my $mapping = $related_meta->{args}->{mapping} || [];
+    
+    my $related_class;
+ 
+    # XXX
+    #warn "This stuff IS all necessary";   # though broken
+#    if ( @$mapping ) 
+#    {
+#        use Data::Dumper;
+#        die Dumper( [ $mapping, $related_meta->{foreign_class}->meta_info( $rel_type ) ] );
+#        $related_class = $related_meta->{foreign_class}->meta_info( $rel_type )->{ $$mapping[0] }->{foreign_class};
+#    }
+#    else 
+#    {
+        $related_class = $related_meta->{foreign_class};
+#    }
+    
+    return ( $related_class, $rel_type ) unless ref( $them );
+    
+    my $accessor = $related_meta->accessor;
+    
+    # multiple objects for has_many
+    my @related_objects = $them->$accessor;
+    
+    return ( $related_objects[0], $rel_type ) if ( @related_objects == 1 and $related_objects[0] );
+    
+    return ( \@related_objects, $rel_type )   if @related_objects > 1;
+    
+    return ( $related_class, $rel_type );
+}
+
+sub _false_related_field_name
+{
+    my ( $me, $them, $real_field_name ) = @_;
+    
+    my $class = $me->_encode_class( $them );
+    my $pk    = $me->_encode_pk( $them );
+    
+    return $real_field_name . $class . $pk;
+}
+
+sub _real_related_field_name
+{
+    my ( $me, $field_name ) = @_;
+
+    # remove any encoded class
+    $field_name =~ s/CDBI_.+_CDBI//;
+    
+    # remove any primary keys
+    $field_name =~ s/PKDATA_.+_PKDATA//;
+    
+    return $field_name;
+}
+
+sub _encode_pk
+{
+    my ( $me, $them ) = @_;
+    
+    return '' unless ref( $them );
+    
+    my @pk = map { $them->get( $_ ) } $them->primary_columns;
+    
+    die "dots in primary key values will confuse _encode_pk and _decode_pk"
+        if grep { /\./ } @pk;
+    
+    my $pk = sprintf 'PKDATA_%s_PKDATA', join( '.', @pk );
+
+    return $pk;
+}
+
+sub _decode_pk
+{
+    my ( $me, $fake_field_name ) = @_;
+    
+    return unless $fake_field_name =~ /PKDATA_(.+)_PKDATA/;
+    
+    my $pv = $1;
+    
+    my @pv = split /\./, $pv;
+    
+    my $class = $me->_decode_class( $fake_field_name );
+    
+    my @pc = map { ''.$_ } $class->primary_columns;
+    
+    my %pk = map { $_ => shift( @pv ) } @pc;
+    
+    return %pk;
+}
+
+sub _decode_class
+{
+    my ( $me, $fake_field_name ) = @_;
+
+    $fake_field_name =~ /^CDBI_(.+)_CDBI/ || die "no class in fake field name $fake_field_name";
+    
+    my $class = $1;
+    
+    $class =~ s/\./::/g;
+    
+    return $class;
+}
+
+sub _encode_class
+{
+    my ( $me, $them ) = @_;
+    
+    my $token = ref( $them ) || $them;
+    
+    $token =~ s/::/./g;
+    
+    return "CDBI_$token\_CDBI";   
+}
+
+=for retiring
+
+sub _make_related_form
+{
+    my ( $me, $them, $orig, $main_args, $related_args ) = @_;
+    
+    # related_class => rel_type
+    my $relatives = delete $main_args->{_relatives_};
+    
+    my %all_rel_types = map { $_ => 1 } keys %$relatives, keys %{ $them->meta_info };
+    
+    my $form = CGI::FormBuilder->new( %$main_args );
+    
+    # { fields  => [ qw( a b c ) ],
+    #   $class  => { fields => [ qw( x y z ) ] },
+    #   $class2 => ...
+    $form->{__cdbi_original_args__} = $orig;
+    
+    # this assumes meta_info only holds data on relationships
+    foreach my $modify ( @BASIC_FORM_MODIFIERS, keys %all_rel_types )
+    {
+        my $form_modify = "form_$modify";
+        
+        $me->$form_modify( $them, $form, $relatives );
+    }
+    
+    return $form;
+}
+
+=cut
+
 =item search_form( %args )
 
 Build a form with inputs that can be fed to search methods (e.g. C<search_where_from_form>). 
@@ -416,9 +803,12 @@ sub form_options
         
         next unless @$series;
         
+        my $value = $them->get( $field ) if ref( $them );
+        
         $form->field( name      => $field,
                       options   => $series,
                       multiple  => $multiple,
+                      value     => $value,
                       );
     }    
 }
@@ -478,6 +868,8 @@ sub form_has_a
     
     my $meta = $them->meta_info( 'has_a' ) || return;
     
+    # XXX
+    #warn "I think this is the wrong pk to use to populate the field value"; # see ???
     my $pk = $them->primary_column;
     
     my @haves = keys %$meta;
@@ -493,7 +885,7 @@ sub form_has_a
                 ref( $them ), $field;
         
         $form->field( name  => $field,
-                      value => $related_object->$pk,
+                      value => $related_object->$pk, # ???
                       );
     }
 }
@@ -582,7 +974,7 @@ sub form_might_have
         my $foreign_pk = $related_class->primary_column;
         
         my $might_have_object = $them->$accessor;
-        my $might_have_object_id = $might_have_object ? $might_have_object->$foreign_pk : '';
+        my $might_have_object_id = $might_have_object ? $might_have_object->$foreign_pk : undef; # was ''
         
         $form->field( name  => $field,
                       value => $might_have_object_id,
@@ -597,8 +989,6 @@ sub _set_field_options
 {
     my ( $me, $them, $form, $field, $rel_type, $field_args ) = @_;
     
-    # Originally this was for passing the 'multiple' flag in has_many, 
-    # but not used for anything at the moment. Seems useful though.
     $field_args ||= {};
 
     return unless exists $form->field->{ $field };
@@ -638,6 +1028,31 @@ sub _field_options
     return \@options;
 }
 
+# this is a better way of working, currently just used in the as_form_with_related parts 
+# of form_* methods, but eventually the other bits will work the same way, rather than 
+# using _set_field_options
+sub _get_field_options
+{
+    my ( $me, $them, $field, $rel_type ) = @_;
+    
+    my $rel = $them->meta_info( $rel_type, $field ) || return;
+    
+    return unless $rel->foreign_class->isa( 'Class::DBI' );
+
+    my $related_class = $rel->foreign_class;
+        
+    my $iter = $related_class->retrieve_all;
+    
+    my @options;
+    
+    while ( my $object = $iter->next )
+    {
+        push @options, [ $object->id, "$object" ];
+    }
+    
+    return \@options;
+}
+
 =back
 
 =head2 Form handling methods
@@ -1156,25 +1571,6 @@ and is probably incomplete. If you come across any failures, you can add suitabl
 
 =cut
 
-our %ValidMap = ( varchar   => 'VALUE',
-                  char      => 'VALUE', # includes MySQL enum and set
-                  blob      => 'VALUE', # includes MySQL text
-                  
-                  integer   => 'INT',
-                  bigint    => 'INT',
-                  smallint  => 'INT',
-                  tinyint   => 'INT',
-                  
-                  date      => 'VALUE',
-                  
-                  # normally you want to skip validating a timestamp column...
-                  #timestamp => 'VALUE',
-                  
-                  double    => 'NUM',
-                  float     => 'NUM',
-                  decimal   => 'NUM',
-                  );    
-                  
 sub _get_type
 {
     my ( $me, $them, $col ) = @_;
